@@ -1,7 +1,7 @@
 /*
  * qrencode - QR Code encoder
  *
- * Copyright (C) 2006, 2007, 2008 Kentaro Fukuchi <fukuchi@megaui.net>
+ * Copyright (C) 2006-2012 Kentaro Fukuchi <kentaro@fukuchi.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,83 +18,137 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 
 #include "qrencode.h"
-#include "qrencode_inner.h"
 #include "qrspec.h"
+#include "mqrspec.h"
 #include "bitstream.h"
 #include "qrinput.h"
 #include "rscode.h"
 #include "split.h"
 #include "mask.h"
+#include "mmask.h"
 
 /******************************************************************************
  * Raw code
  *****************************************************************************/
 
-static void RSblock_init(RSblock *block, int dl, unsigned char *data, int el)
-{
-	RS *rs;
+typedef struct {
+	int dataLength;
+	unsigned char *data;
+	int eccLength;
+	unsigned char *ecc;
+} RSblock;
 
+typedef struct {
+	int version;
+	int dataLength;
+	int eccLength;
+	unsigned char *datacode;
+	unsigned char *ecccode;
+	int b1;
+	int blocks;
+	RSblock *rsblock;
+	int count;
+} QRRawCode;
+
+static void RSblock_initBlock(RSblock *block, int dl, unsigned char *data, int el, unsigned char *ecc, RS *rs)
+{
 	block->dataLength = dl;
 	block->data = data;
 	block->eccLength = el;
-	block->ecc = (unsigned char *)malloc(el);
+	block->ecc = ecc;
 
-	rs = init_rs(8, 0x11d, 0, 1, el, 255 - dl - el);
-	encode_rs_char(rs, data, block->ecc);
+	encode_rs_char(rs, data, ecc);
 }
 
-QRRawCode *QRraw_new(QRinput *input)
+static int RSblock_init(RSblock *blocks, int spec[5], unsigned char *data, unsigned char *ecc)
 {
-	QRRawCode *raw;
-	int *spec;
 	int i;
-	RSblock *rsblock;
-	unsigned char *p;
+	RSblock *block;
+	unsigned char *dp, *ep;
+	RS *rs;
+	int el, dl;
 
-	p = QRinput_getByteStream(input);
-	if(p == NULL) {
-		return NULL;
+	dl = QRspec_rsDataCodes1(spec);
+	el = QRspec_rsEccCodes1(spec);
+	rs = init_rs(8, 0x11d, 0, 1, el, 255 - dl - el);
+	if(rs == NULL) return -1;
+
+	block = blocks;
+	dp = data;
+	ep = ecc;
+	for(i=0; i<QRspec_rsBlockNum1(spec); i++) {
+		RSblock_initBlock(block, dl, dp, el, ep, rs);
+		dp += dl;
+		ep += el;
+		block++;
 	}
 
+	if(QRspec_rsBlockNum2(spec) == 0) return 0;
+
+	dl = QRspec_rsDataCodes2(spec);
+	el = QRspec_rsEccCodes2(spec);
+	rs = init_rs(8, 0x11d, 0, 1, el, 255 - dl - el);
+	if(rs == NULL) return -1;
+	for(i=0; i<QRspec_rsBlockNum2(spec); i++) {
+		RSblock_initBlock(block, dl, dp, el, ep, rs);
+		dp += dl;
+		ep += el;
+		block++;
+	}
+
+	return 0;
+}
+
+__STATIC void QRraw_free(QRRawCode *raw);
+__STATIC QRRawCode *QRraw_new(QRinput *input)
+{
+	QRRawCode *raw;
+	int spec[5], ret;
+
 	raw = (QRRawCode *)malloc(sizeof(QRRawCode));
-	raw->datacode = p;
-	spec = QRspec_getEccSpec(input->version, input->level);
-	if(spec == NULL) {
+	if(raw == NULL) return NULL;
+
+	raw->datacode = QRinput_getByteStream(input);
+	if(raw->datacode == NULL) {
 		free(raw);
 		return NULL;
 	}
+
+	QRspec_getEccSpec(input->version, input->level, spec);
+
 	raw->version = input->version;
-	raw->blocks = QRspec_rsBlockNum(spec);
-	raw->rsblock = (RSblock *)malloc(sizeof(RSblock) * raw->blocks);
-
-	rsblock = raw->rsblock;
-	p = raw->datacode;
-	for(i=0; i<QRspec_rsBlockNum1(spec); i++) {
-		RSblock_init(rsblock, QRspec_rsDataCodes1(spec), p,
-						QRspec_rsEccCodes1(spec));
-		p += QRspec_rsDataCodes1(spec);
-		rsblock++;
-	}
-	for(i=0; i<QRspec_rsBlockNum2(spec); i++) {
-		RSblock_init(rsblock, QRspec_rsDataCodes2(spec), p,
-						QRspec_rsEccCodes2(spec));
-		p += QRspec_rsDataCodes2(spec);
-		rsblock++;
-	}
-
 	raw->b1 = QRspec_rsBlockNum1(spec);
-	raw->dataLength = QRspec_rsBlockNum1(spec) * QRspec_rsDataCodes1(spec)
-					+ QRspec_rsBlockNum2(spec) * QRspec_rsDataCodes2(spec);
-	raw->eccLength = QRspec_rsBlockNum(spec) * QRspec_rsEccCodes1(spec);
-	raw->count = 0;
+	raw->dataLength = QRspec_rsDataLength(spec);
+	raw->eccLength = QRspec_rsEccLength(spec);
+	raw->ecccode = (unsigned char *)malloc(raw->eccLength);
+	if(raw->ecccode == NULL) {
+		free(raw->datacode);
+		free(raw);
+		return NULL;
+	}
 
-	free(spec);
+	raw->blocks = QRspec_rsBlockNum(spec);
+	raw->rsblock = (RSblock *)calloc(raw->blocks, sizeof(RSblock));
+	if(raw->rsblock == NULL) {
+		QRraw_free(raw);
+		return NULL;
+	}
+	ret = RSblock_init(raw->rsblock, spec, raw->datacode, raw->ecccode);
+	if(ret < 0) {
+		QRraw_free(raw);
+		return NULL;
+	}
+
+	raw->count = 0;
 
 	return raw;
 }
@@ -105,7 +159,7 @@ QRRawCode *QRraw_new(QRinput *input)
  * @param raw raw code.
  * @return code
  */
-unsigned char QRraw_getCode(QRRawCode *raw)
+__STATIC unsigned char QRraw_getCode(QRRawCode *raw)
 {
 	int col, row;
 	unsigned char ret;
@@ -113,7 +167,7 @@ unsigned char QRraw_getCode(QRRawCode *raw)
 	if(raw->count < raw->dataLength) {
 		row = raw->count % raw->blocks;
 		col = raw->count / raw->blocks;
-		if(col >= raw->rsblock[row].dataLength) {
+		if(col >= raw->rsblock[0].dataLength) {
 			row += raw->b1;
 		}
 		ret = raw->rsblock[row].data[col];
@@ -128,18 +182,107 @@ unsigned char QRraw_getCode(QRRawCode *raw)
 	return ret;
 }
 
-void QRraw_free(QRRawCode *raw)
+__STATIC void QRraw_free(QRRawCode *raw)
 {
-	int i;
-
-	free(raw->datacode);
-	for(i=0; i<raw->blocks; i++) {
-		free(raw->rsblock[i].ecc);
+	if(raw != NULL) {
+		free(raw->datacode);
+		free(raw->ecccode);
+		free(raw->rsblock);
+		free(raw);
 	}
-	free(raw->rsblock);
-	free(raw);
 }
 
+/******************************************************************************
+ * Raw code for Micro QR Code
+ *****************************************************************************/
+
+typedef struct {
+	int version;
+	int dataLength;
+	int eccLength;
+	unsigned char *datacode;
+	unsigned char *ecccode;
+	RSblock *rsblock;
+	int oddbits;
+	int count;
+} MQRRawCode;
+
+__STATIC void MQRraw_free(MQRRawCode *raw);
+__STATIC MQRRawCode *MQRraw_new(QRinput *input)
+{
+	MQRRawCode *raw;
+	RS *rs;
+
+	raw = (MQRRawCode *)malloc(sizeof(MQRRawCode));
+	if(raw == NULL) return NULL;
+
+	raw->version = input->version;
+	raw->dataLength = MQRspec_getDataLength(input->version, input->level);
+	raw->eccLength = MQRspec_getECCLength(input->version, input->level);
+	raw->oddbits = raw->dataLength * 8 - MQRspec_getDataLengthBit(input->version, input->level);
+	raw->datacode = QRinput_getByteStream(input);
+	if(raw->datacode == NULL) {
+		free(raw);
+		return NULL;
+	}
+	raw->ecccode = (unsigned char *)malloc(raw->eccLength);
+	if(raw->ecccode == NULL) {
+		free(raw->datacode);
+		free(raw);
+		return NULL;
+	}
+
+	raw->rsblock = (RSblock *)calloc(1, sizeof(RSblock));
+	if(raw->rsblock == NULL) {
+		MQRraw_free(raw);
+		return NULL;
+	}
+
+	rs = init_rs(8, 0x11d, 0, 1, raw->eccLength, 255 - raw->dataLength - raw->eccLength);
+	if(rs == NULL) {
+		MQRraw_free(raw);
+		return NULL;
+	}
+
+	RSblock_initBlock(raw->rsblock, raw->dataLength, raw->datacode, raw->eccLength, raw->ecccode, rs);
+
+	raw->count = 0;
+
+	return raw;
+}
+
+/**
+ * Return a code (byte).
+ * This function can be called iteratively.
+ * @param raw raw code.
+ * @return code
+ */
+__STATIC unsigned char MQRraw_getCode(MQRRawCode *raw)
+{
+	unsigned char ret;
+
+	if(raw->count < raw->dataLength) {
+		ret = raw->datacode[raw->count];
+	} else if(raw->count < raw->dataLength + raw->eccLength) {
+		ret = raw->ecccode[raw->count - raw->dataLength];
+	} else {
+		return 0;
+	}
+	raw->count++;
+	return ret;
+}
+
+__STATIC void MQRraw_free(MQRRawCode *raw)
+{
+	if(raw != NULL) {
+		free(raw->datacode);
+		free(raw->ecccode);
+		free(raw->rsblock);
+		free(raw);
+	}
+}
+
+
 /******************************************************************************
  * Frame filling
  *****************************************************************************/
@@ -150,19 +293,22 @@ typedef struct {
 	int x, y;
 	int dir;
 	int bit;
+	int mqr;
 } FrameFiller;
 
-static FrameFiller *FrameFiller_new(int width, unsigned char *frame)
+static FrameFiller *FrameFiller_new(int width, unsigned char *frame, int mqr)
 {
 	FrameFiller *filler;
 
 	filler = (FrameFiller *)malloc(sizeof(FrameFiller));
+	if(filler == NULL) return NULL;
 	filler->width = width;
 	filler->frame = frame;
 	filler->x = width - 1;
 	filler->y = width - 1;
 	filler->dir = -1;
 	filler->bit = -1;
+	filler->mqr = mqr;
 
 	return filler;
 }
@@ -196,7 +342,7 @@ static unsigned char *FrameFiller_next(FrameFiller *filler)
 			y = 0;
 			x -= 2;
 			filler->dir = 1;
-			if(x == 6) {
+			if(!filler->mqr && x == 6) {
 				x--;
 				y = 9;
 			}
@@ -206,7 +352,7 @@ static unsigned char *FrameFiller_next(FrameFiller *filler)
 			y = w - 1;
 			x -= 2;
 			filler->dir = -1;
-			if(x == 6) {
+			if(!filler->mqr && x == 6) {
 				x--;
 				y -= 8;
 			}
@@ -224,104 +370,81 @@ static unsigned char *FrameFiller_next(FrameFiller *filler)
 	return &p[y * w + x];
 }
 
-#if 0
-unsigned char *FrameFiller_fillerTest(int version)
+#ifdef WITH_TESTS
+extern unsigned char *FrameFiller_test(int version)
 {
-	int width, length;
+	int width;
 	unsigned char *frame, *p;
 	FrameFiller *filler;
-	int i, j;
-	unsigned char cl = 1;
-	unsigned char ch = 0;
+	int i, length;
 
 	width = QRspec_getWidth(version);
 	frame = QRspec_newFrame(version);
-	filler = FrameFiller_new(width, frame);
-	length = QRspec_getDataLength(version, QR_ECLEVEL_L)
-			+ QRspec_getECCLength(version, QR_ECLEVEL_L);
-
-	for(i=0; i<length; i++) {
-		for(j=0; j<8; j++) {
-			p = FrameFiller_next(filler);
-			*p = ch | cl;
-			cl++;
-			if(cl == 9) {
-				cl = 1;
-				ch += 0x10;
-			}
-		}
-	}
-	length = QRspec_getRemainder(version);
-	for(i=0; i<length; i++) {
-		p = FrameFiller_next(filler);
-		*p = 0xa0;
-	}
-	p = FrameFiller_next(filler);
-	free(filler);
-	if(p != NULL) {
+	if(frame == NULL) return NULL;
+	filler = FrameFiller_new(width, frame, 0);
+	if(filler == NULL) {
+		free(frame);
 		return NULL;
 	}
+	length = QRspec_getDataLength(version, QR_ECLEVEL_L) * 8
+	       + QRspec_getECCLength(version, QR_ECLEVEL_L) * 8
+		   + QRspec_getRemainder(version);
+	for(i=0; i<length; i++) {
+		p = FrameFiller_next(filler);
+		if(p == NULL) {
+			free(filler);
+			free(frame);
+			return NULL;
+		}
+		*p = (unsigned char)(i & 0x7f) | 0x80;
+	}
+	free(filler);
+	return frame;
+}
 
+extern unsigned char *FrameFiller_testMQR(int version)
+{
+	int width;
+	unsigned char *frame, *p;
+	FrameFiller *filler;
+	int i, length;
+
+	width = MQRspec_getWidth(version);
+	frame = MQRspec_newFrame(version);
+	if(frame == NULL) return NULL;
+	filler = FrameFiller_new(width, frame, 1);
+	if(filler == NULL) {
+		free(frame);
+		return NULL;
+	}
+	length = MQRspec_getDataLengthBit(version, QR_ECLEVEL_L)
+	       + MQRspec_getECCLength(version, QR_ECLEVEL_L) * 8;
+	for(i=0; i<length; i++) {
+		p = FrameFiller_next(filler);
+		if(p == NULL) {
+			fprintf(stderr, "Frame filler run over the frame!\n");
+			free(filler);
+			return frame;
+		}
+		*p = (unsigned char)(i & 0x7f) | 0x80;
+	}
+	free(filler);
 	return frame;
 }
 #endif
 
-/******************************************************************************
- * Format information
- *****************************************************************************/
-
-int QRcode_writeFormatInformation(int width, unsigned char *frame, int mask, QRecLevel level)
-{
-	unsigned int format;
-	unsigned char v;
-	int i;
-	int blacks = 0;
-
-	format =  QRspec_getFormatInfo(mask, level);
-
-	for(i=0; i<8; i++) {
-		if(format & 1) {
-			blacks += 2;
-			v = 0x85;
-		} else {
-			v = 0x84;
-		}
-		frame[width * 8 + width - 1 - i] = v;
-		if(i < 6) {
-			frame[width * i + 8] = v;
-		} else {
-			frame[width * (i + 1) + 8] = v;
-		}
-		format= format >> 1;
-	}
-	for(i=0; i<7; i++) {
-		if(format & 1) {
-			blacks += 2;
-			v = 0x85;
-		} else {
-			v = 0x84;
-		}
-		frame[width * (width - 7 + i) + 8] = v;
-		if(i == 0) {
-			frame[width * 8 + 7] = v;
-		} else {
-			frame[width * 8 + 6 - i] = v;
-		}
-		format= format >> 1;
-	}
-
-	return blacks;
-}
 
 /******************************************************************************
  * QR-code encoding
  *****************************************************************************/
 
-static QRcode *QRcode_new(int version, int width, unsigned char *data)
+__STATIC QRcode *QRcode_new(int version, int width, unsigned char *data)
 {
 	QRcode *qrcode;
 
 	qrcode = (QRcode *)malloc(sizeof(QRcode));
+	if(qrcode == NULL) return NULL;
+
 	qrcode->version = version;
 	qrcode->width = width;
 	qrcode->data = data;
@@ -331,28 +454,30 @@ static QRcode *QRcode_new(int version, int width, unsigned char *data)
 
 void QRcode_free(QRcode *qrcode)
 {
-	if(qrcode == NULL) return;
-
-	if(qrcode->data != NULL) {
+	if(qrcode != NULL) {
 		free(qrcode->data);
+		free(qrcode);
 	}
-	free(qrcode);
 }
 
-QRcode *QRcode_encodeMask(QRinput *input, int mask)
+__STATIC QRcode *QRcode_encodeMask(QRinput *input, int mask)
 {
 	int width, version;
 	QRRawCode *raw;
 	unsigned char *frame, *masked, *p, code, bit;
 	FrameFiller *filler;
 	int i, j;
-	QRcode *qrcode;
+	QRcode *qrcode = NULL;
 
+	if(input->mqr) {
+		errno = EINVAL;
+		return NULL;
+	}
 	if(input->version < 0 || input->version > QRSPEC_VERSION_MAX) {
 		errno = EINVAL;
 		return NULL;
 	}
-	if(input->level < QR_ECLEVEL_L || input->level > QR_ECLEVEL_H) {
+	if(input->level > QR_ECLEVEL_H) {
 		errno = EINVAL;
 		return NULL;
 	}
@@ -363,7 +488,16 @@ QRcode *QRcode_encodeMask(QRinput *input, int mask)
 	version = raw->version;
 	width = QRspec_getWidth(version);
 	frame = QRspec_newFrame(version);
-	filler = FrameFiller_new(width, frame);
+	if(frame == NULL) {
+		QRraw_free(raw);
+		return NULL;
+	}
+	filler = FrameFiller_new(width, frame, 0);
+	if(filler == NULL) {
+		QRraw_free(raw);
+		free(frame);
+		return NULL;
+	}
 
 	/* inteleaved data and ecc codes */
 	for(i=0; i<raw->dataLength + raw->eccLength; i++) {
@@ -371,46 +505,160 @@ QRcode *QRcode_encodeMask(QRinput *input, int mask)
 		bit = 0x80;
 		for(j=0; j<8; j++) {
 			p = FrameFiller_next(filler);
+			if(p == NULL)  goto EXIT;
 			*p = 0x02 | ((bit & code) != 0);
 			bit = bit >> 1;
 		}
 	}
 	QRraw_free(raw);
+	raw = NULL;
 	/* remainder bits */
 	j = QRspec_getRemainder(version);
 	for(i=0; i<j; i++) {
 		p = FrameFiller_next(filler);
+		if(p == NULL)  goto EXIT;
 		*p = 0x02;
 	}
-	free(filler);
+
 	/* masking */
-	if(mask < 0) {
+	if(mask == -2) { // just for debug purpose
+		masked = (unsigned char *)malloc(width * width);
+		memcpy(masked, frame, width * width);
+	} else if(mask < 0) {
 		masked = Mask_mask(width, frame, input->level);
 	} else {
-		masked = Mask_makeMask(width, frame, mask);
-		QRcode_writeFormatInformation(width, masked, mask, input->level);
+		masked = Mask_makeMask(width, frame, mask, input->level);
+	}
+	if(masked == NULL) {
+		goto EXIT;
 	}
 	qrcode = QRcode_new(version, width, masked);
 
+EXIT:
+	QRraw_free(raw);
+	free(filler);
 	free(frame);
+	return qrcode;
+}
 
+__STATIC QRcode *QRcode_encodeMaskMQR(QRinput *input, int mask)
+{
+	int width, version;
+	MQRRawCode *raw;
+	unsigned char *frame, *masked, *p, code, bit;
+	FrameFiller *filler;
+	int i, j;
+	QRcode *qrcode = NULL;
+
+	if(!input->mqr) {
+		errno = EINVAL;
+		return NULL;
+	}
+	if(input->version <= 0 || input->version > MQRSPEC_VERSION_MAX) {
+		errno = EINVAL;
+		return NULL;
+	}
+	if(input->level > QR_ECLEVEL_Q) {
+		errno = EINVAL;
+		return NULL;
+	}
+
+	raw = MQRraw_new(input);
+	if(raw == NULL) return NULL;
+
+	version = raw->version;
+	width = MQRspec_getWidth(version);
+	frame = MQRspec_newFrame(version);
+	if(frame == NULL) {
+		MQRraw_free(raw);
+		return NULL;
+	}
+	filler = FrameFiller_new(width, frame, 1);
+	if(filler == NULL) {
+		MQRraw_free(raw);
+		free(frame);
+		return NULL;
+	}
+
+	/* inteleaved data and ecc codes */
+	for(i=0; i<raw->dataLength + raw->eccLength; i++) {
+		code = MQRraw_getCode(raw);
+		if(raw->oddbits && i == raw->dataLength - 1) {
+			bit = 1 << raw->oddbits;
+			for(j=0; j<raw->oddbits; j++) {
+				p = FrameFiller_next(filler);
+				if(p == NULL) goto EXIT;
+				*p = 0x02 | ((bit & code) != 0);
+				bit = bit >> 1;
+			}
+		} else {
+			bit = 0x80;
+			for(j=0; j<8; j++) {
+				p = FrameFiller_next(filler);
+				if(p == NULL) goto EXIT;
+				*p = 0x02 | ((bit & code) != 0);
+				bit = bit >> 1;
+			}
+		}
+	}
+	MQRraw_free(raw);
+	raw = NULL;
+
+	/* masking */
+	if(mask < 0) {
+		masked = MMask_mask(version, frame, input->level);
+	} else {
+		masked = MMask_makeMask(version, frame, mask, input->level);
+	}
+	if(masked == NULL) {
+		goto EXIT;
+	}
+
+	qrcode = QRcode_new(version, width, masked);
+
+EXIT:
+	MQRraw_free(raw);
+	free(filler);
+	free(frame);
 	return qrcode;
 }
 
 QRcode *QRcode_encodeInput(QRinput *input)
 {
-	return QRcode_encodeMask(input, -1);
+	if(input->mqr) {
+		return QRcode_encodeMaskMQR(input, -1);
+	} else {
+		return QRcode_encodeMask(input, -1);
+	}
 }
 
-QRcode *QRcode_encodeString8bit(const char *string, int version, QRecLevel level)
+static QRcode *QRcode_encodeStringReal(const char *string, int version, QRecLevel level, int mqr, QRencodeMode hint, int casesensitive)
 {
 	QRinput *input;
 	QRcode *code;
+	int ret;
 
-	input = QRinput_new2(version, level);
+	if(string == NULL) {
+		errno = EINVAL;
+		return NULL;
+	}
+	if(hint != QR_MODE_8 && hint != QR_MODE_KANJI) {
+		errno = EINVAL;
+		return NULL;
+	}
+
+	if(mqr) {
+		input = QRinput_newMQR(version, level);
+	} else {
+		input = QRinput_new2(version, level);
+	}
 	if(input == NULL) return NULL;
 
-	QRinput_append(input, QR_MODE_8, strlen(string), (unsigned char *)string);
+	ret = Split_splitStringToQRinput(string, input, hint, casesensitive);
+	if(ret < 0) {
+		QRinput_free(input);
+		return NULL;
+	}
 	code = QRcode_encodeInput(input);
 	QRinput_free(input);
 
@@ -418,31 +666,73 @@ QRcode *QRcode_encodeString8bit(const char *string, int version, QRecLevel level
 }
 
 QRcode *QRcode_encodeString(const char *string, int version, QRecLevel level, QRencodeMode hint, int casesensitive)
+{
+	return QRcode_encodeStringReal(string, version, level, 0, hint, casesensitive);
+}
+
+QRcode *QRcode_encodeStringMQR(const char *string, int version, QRecLevel level, QRencodeMode hint, int casesensitive)
+{
+	return QRcode_encodeStringReal(string, version, level, 1, hint, casesensitive);
+}
+
+static QRcode *QRcode_encodeDataReal(const unsigned char *data, int length, int version, QRecLevel level, int mqr)
 {
 	QRinput *input;
 	QRcode *code;
 	int ret;
 
-	if(hint != QR_MODE_8 && hint != QR_MODE_KANJI) {
+	if(data == NULL || length == 0) {
 		errno = EINVAL;
 		return NULL;
 	}
 
-	input = QRinput_new2(version, level);
+	if(mqr) {
+		input = QRinput_newMQR(version, level);
+	} else {
+		input = QRinput_new2(version, level);
+	}
 	if(input == NULL) return NULL;
 
-	ret = Split_splitStringToQRinput(string, input, hint, casesensitive);
+	ret = QRinput_append(input, QR_MODE_8, length, data);
 	if(ret < 0) {
 		QRinput_free(input);
 		return NULL;
 	}
-
 	code = QRcode_encodeInput(input);
 	QRinput_free(input);
 
 	return code;
 }
 
+QRcode *QRcode_encodeData(int size, const unsigned char *data, int version, QRecLevel level)
+{
+	return QRcode_encodeDataReal(data, size, version, level, 0);
+}
+
+QRcode *QRcode_encodeString8bit(const char *string, int version, QRecLevel level)
+{
+	if(string == NULL) {
+		errno = EINVAL;
+		return NULL;
+	}
+	return QRcode_encodeDataReal((unsigned char *)string, strlen(string), version, level, 0);
+}
+
+QRcode *QRcode_encodeDataMQR(int size, const unsigned char *data, int version, QRecLevel level)
+{
+	return QRcode_encodeDataReal(data, size, version, level, 1);
+}
+
+QRcode *QRcode_encodeString8bitMQR(const char *string, int version, QRecLevel level)
+{
+	if(string == NULL) {
+		errno = EINVAL;
+		return NULL;
+	}
+	return QRcode_encodeDataReal((unsigned char *)string, strlen(string), version, level, 1);
+}
+
+
 /******************************************************************************
  * Structured QR-code encoding
  *****************************************************************************/
@@ -462,8 +752,10 @@ static QRcode_List *QRcode_List_newEntry(void)
 
 static void QRcode_List_freeEntry(QRcode_List *entry)
 {
-	if(entry->code != NULL) QRcode_free(entry->code);
-	free(entry);
+	if(entry != NULL) {
+		QRcode_free(entry->code);
+		free(entry);
+	}
 }
 
 void QRcode_List_free(QRcode_List *qrlist)
@@ -508,25 +800,32 @@ QRcode_List *QRcode_encodeInputStructured(QRinput_Struct *s)
 {
 	QRcode_List *head = NULL;
 	QRcode_List *tail = NULL;
+	QRcode_List *entry;
 	QRinput_InputList *list = s->head;
 
 	while(list != NULL) {
 		if(head == NULL) {
-			head = QRcode_List_newEntry();
+			entry = QRcode_List_newEntry();
+			if(entry == NULL) goto ABORT;
+			head = entry;
 			tail = head;
 		} else {
-			tail->next = QRcode_List_newEntry();
+			entry = QRcode_List_newEntry();
+			if(entry == NULL) goto ABORT;
+			tail->next = entry;
 			tail = tail->next;
 		}
 		tail->code = QRcode_encodeInput(list->input);
 		if(tail->code == NULL) {
-			QRcode_List_free(head);
-			return NULL;
+			goto ABORT;
 		}
 		list = list->next;
 	}
 
 	return head;
+ABORT:
+	QRcode_List_free(head);
+	return NULL;
 }
 
 static QRcode_List *QRcode_encodeInputToStructured(QRinput *input)
@@ -543,18 +842,32 @@ static QRcode_List *QRcode_encodeInputToStructured(QRinput *input)
 	return codes;
 }
 
-QRcode_List *QRcode_encodeString8bitStructured(const char *string, int version, QRecLevel level)
+static QRcode_List *QRcode_encodeDataStructuredReal(
+	int size, const unsigned char *data,
+	int version, QRecLevel level,
+	int eightbit, QRencodeMode hint, int casesensitive)
 {
 	QRinput *input;
 	QRcode_List *codes;
 	int ret;
 
-	if(version <= 0) return NULL;
+	if(version <= 0) {
+		errno = EINVAL;
+		return NULL;
+	}
+	if(!eightbit && (hint != QR_MODE_8 && hint != QR_MODE_KANJI)) {
+		errno = EINVAL;
+		return NULL;
+	}
 
 	input = QRinput_new2(version, level);
 	if(input == NULL) return NULL;
 
-	ret = QRinput_append(input, QR_MODE_8, strlen(string), (unsigned char *)string);
+	if(eightbit) {
+		ret = QRinput_append(input, QR_MODE_8, size, data);
+	} else {
+		ret = Split_splitStringToQRinput((char *)data, input, hint, casesensitive);
+	}
 	if(ret < 0) {
 		QRinput_free(input);
 		return NULL;
@@ -565,27 +878,52 @@ QRcode_List *QRcode_encodeString8bitStructured(const char *string, int version,
 	return codes;
 }
 
+QRcode_List *QRcode_encodeDataStructured(int size, const unsigned char *data, int version, QRecLevel level) {
+	return QRcode_encodeDataStructuredReal(size, data, version, level, 1, QR_MODE_NUL, 0);
+}
+
+QRcode_List *QRcode_encodeString8bitStructured(const char *string, int version, QRecLevel level) {
+	if(string == NULL) {
+		errno = EINVAL;
+		return NULL;
+	}
+	return QRcode_encodeDataStructured(strlen(string), (unsigned char *)string, version, level);
+}
+
 QRcode_List *QRcode_encodeStringStructured(const char *string, int version, QRecLevel level, QRencodeMode hint, int casesensitive)
 {
-	QRinput *input;
-	QRcode_List *codes;
-	int ret;
-
-	if(version <= 0) return NULL;
-	if(hint != QR_MODE_8 && hint != QR_MODE_KANJI) {
+	if(string == NULL) {
+		errno = EINVAL;
 		return NULL;
 	}
+	return QRcode_encodeDataStructuredReal(strlen(string), (unsigned char *)string, version, level, 0, hint, casesensitive);
+}
 
-	input = QRinput_new2(version, level);
-	if(input == NULL) return NULL;
+/******************************************************************************
+ * System utilities
+ *****************************************************************************/
 
-	ret = Split_splitStringToQRinput(string, input, hint, casesensitive);
-	if(ret < 0) {
-		QRinput_free(input);
-		return NULL;
+void QRcode_APIVersion(int *major_version, int *minor_version, int *micro_version)
+{
+	if(major_version != NULL) {
+		*major_version = MAJOR_VERSION;
 	}
-	codes = QRcode_encodeInputToStructured(input);
-	QRinput_free(input);
+	if(minor_version != NULL) {
+		*minor_version = MINOR_VERSION;
+	}
+	if(micro_version != NULL) {
+		*micro_version = MICRO_VERSION;
+	}
+}
+
+char *QRcode_APIVersionString(void)
+{
+	return VERSION;
+}
 
-	return codes;
+void QRcode_clearCache(void)
+{
+	QRspec_clearCache();
+	MQRspec_clearCache();
+	free_rs_cache();
 }
